﻿using System;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Threading;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.StorageClient;

namespace CloudDelete
{
    public class WorkerRole : RoleEntryPoint
    {
        private CloudBlobClient blob_cli;
        private CloudQueueClient que_cli;

        public override void Run()
        {
            var account = CloudStorageAccount.FromConfigurationSetting("MyStorage");
            blob_cli = account.CreateCloudBlobClient();
            que_cli = account.CreateCloudQueueClient();
            var queue = que_cli.GetQueueReference("delimages");
            queue.CreateIfNotExist();
            while (true)
            {
                CloudQueueMessage msg = queue.GetMessage();
                try
                {
                    if (msg != null)
                    {
                        this.DeleteImages(msg.AsString);
                        Trace.TraceInformation("Deleted Image:" + msg.AsString);
                        queue.DeleteMessage(msg);
                        Thread.Sleep(5000);
                    }
                    else
                    {
                        Thread.Sleep(10000);
                    }
                }
                catch (Exception e)
                {
                    Trace.TraceError("Error Occured:" + e.Message);
                    Thread.Sleep(10000);
                }
            }
        }

        private void DeleteImages(string uri)
        {
            var c_thumb = blob_cli.GetContainerReference("quethumb");
            var b_thumb = c_thumb.GetBlobReference(uri);
            b_thumb.DeleteIfExists();
            var c_org = blob_cli.GetContainerReference("queorg");
            var b_org = c_org.GetBlobReference(uri);
            b_org.DeleteIfExists();
        }

        public override bool OnStart()
        {
            // Set the maximum number of concurrent connections 
            ServicePointManager.DefaultConnectionLimit = 12;

            DiagnosticMonitor.Start("DiagnosticsConnectionString");

            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.
            //RoleEnvironment.Changing += RoleEnvironmentChanging;

            CloudStorageAccount.SetConfigurationSettingPublisher((configName, configSetter) =>
            {
                configSetter(RoleEnvironment.GetConfigurationSettingValue(configName));

                RoleEnvironment.Changed += (sender, arg) =>
                {
                    if (arg.Changes.OfType<RoleEnvironmentConfigurationSettingChange>()
                        .Any((change) => (change.ConfigurationSettingName == configName)))
                    {
                        if (!configSetter(RoleEnvironment.GetConfigurationSettingValue(configName)))
                        {
                            RoleEnvironment.RequestRecycle();
                        }
                    }
                };
            });
            return base.OnStart();
        }

        private void RoleEnvironmentChanging(object sender, RoleEnvironmentChangingEventArgs e)
        {
            // If a configuration setting is changing
            if (e.Changes.Any(change => change is RoleEnvironmentConfigurationSettingChange))
            {
                // Set e.Cancel to true to restart this role instance
                e.Cancel = true;
            }
        }
    }
}
