﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Net;
using System.Threading;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.StorageClient;


namespace CloudThumbs
{
    public class WorkerRole : RoleEntryPoint
    {
        private CloudBlobClient blob_cli;
        private CloudQueueClient que_cli;

        public override void Run()
        {
            var account = CloudStorageAccount.FromConfigurationSetting("MyStorage");
            blob_cli = account.CreateCloudBlobClient();
            que_cli = account.CreateCloudQueueClient();
            var queue = que_cli.GetQueueReference("thumbnails");
            queue.CreateIfNotExist();
            while (true)
            {
                CloudQueueMessage msg = queue.GetMessage();
                try
                {
                    if (msg != null)
                    {
                        this.CreateThumbnail(msg);
                        queue.DeleteMessage(msg);
                        Trace.TraceInformation("Created thumbnail:" + msg.AsString);
                        Thread.Sleep(5000);
                    }
                    else
                    {
                        Thread.Sleep(10000);
                    }
                }
                catch (Exception e)
                {
                    Trace.TraceError("Error Occured:" + e.Message);
                    Thread.Sleep(10000);
                }
            }
        }

        private void CreateThumbnail(CloudQueueMessage msg)
        {
            var org_c = blob_cli.GetContainerReference("queorg");
            var thumb_c = blob_cli.GetContainerReference("quethumb");
            var blobname = msg.AsString;
            var org = org_c.GetBlobReference(blobname);
            var thumb = thumb_c.GetBlobReference(blobname);

            using (var input = new MemoryStream())
            using (var output = new MemoryStream())
            {
                org.DownloadToStream(input);
                using (var org_img = Image.FromStream(input))
                {
                    int t_width = 50;
                    int t_height = 50;
                    if (org_img.Width > org_img.Height)
                    {
                        t_height = 50 * org_img.Height / org_img.Width;
                    }
                    else
                    {
                        t_height = 50 * org_img.Width / org_img.Height;
                    }
                    using (var thumb_img = org_img.GetThumbnailImage(t_width, t_height, null, IntPtr.Zero))
                    {
                        thumb_img.Save(output, ImageFormat.Jpeg);
                        output.Seek(0, SeekOrigin.Begin);
                    }
                }
                thumb.UploadFromStream(output);
            }
        }

        public override bool OnStart()
        {
            // Set the maximum number of concurrent connections 
            ServicePointManager.DefaultConnectionLimit = 12;

            DiagnosticMonitor.Start("DiagnosticsConnectionString");

            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.
            //RoleEnvironment.Changing += RoleEnvironmentChanging;

            CloudStorageAccount.SetConfigurationSettingPublisher((configName, configSetter) =>
            {
                configSetter(RoleEnvironment.GetConfigurationSettingValue(configName));

                RoleEnvironment.Changed += (sender, arg) =>
                {
                    if (arg.Changes.OfType<RoleEnvironmentConfigurationSettingChange>()
                        .Any((change) => (change.ConfigurationSettingName == configName)))
                    {
                        if (!configSetter(RoleEnvironment.GetConfigurationSettingValue(configName)))
                        {
                            RoleEnvironment.RequestRecycle();
                        }
                    }
                };
            });

            return base.OnStart();
        }

        private void RoleEnvironmentChanging(object sender, RoleEnvironmentChangingEventArgs e)
        {
            // If a configuration setting is changing
            if (e.Changes.Any(change => change is RoleEnvironmentConfigurationSettingChange))
            {
                // Set e.Cancel to true to restart this role instance
                e.Cancel = true;
            }
        }
    }
}
