﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;

namespace CloudUI.Models
{
    public class AlbumImage
    {
        private CloudBlobClient _cli;

        public AlbumImage()
        {
            CloudStorageAccount account = CloudStorageAccount.FromConfigurationSetting("MyStorage");
            _cli = account.CreateCloudBlobClient();
            _cli.RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromMilliseconds(500));
        }

        public List<BlobContainer> GetContainers()
        {
            return (from c in _cli.ListContainers("img", ContainerListingDetails.All)
                    select new BlobContainer
                    {
                        Name = c.Name,
                        DisplayName = HttpUtility.UrlDecode(c.Metadata["Name"])
                    }).ToList();

        }

        public void UploadImage(String containerName, HttpPostedFile fl, String memo)
        {
            var container = _cli.GetContainerReference(containerName);
            var name = String.Format("{0}_{1}",
                    DateTime.UtcNow.ToString("yyyyMMddHHmmss"),
                    Guid.NewGuid().ToString());
            var blob = container.GetBlobReference(name);
            blob.UploadFromStream(fl.InputStream);
            blob.Properties.ContentType = fl.ContentType;
            blob.SetProperties();
            blob.Metadata.Add("memo", HttpUtility.UrlEncode(memo));
            blob.SetMetadata();
        }

        public List<BlobData> GetImages(String containerName)
        {
            var container = _cli.GetContainerReference(containerName);
            var opt = new BlobRequestOptions
            {
                BlobListingDetails = BlobListingDetails.Metadata
            };
            return (from b in container.ListBlobs(opt)
                    let props = ((CloudBlob)b).Properties
                    select new BlobData()
                    {
                        Uri = b.Uri.ToString(),
                        Memo = HttpUtility.UrlDecode(((CloudBlob)b).Metadata["memo"]),
                        Size = props.Length / 1024,
                        Updated = props.LastModifiedUtc.AddHours(9)
                    }).ToList();
        }

        public void DeleteImage(String uri)
        {
            var blob = _cli.GetBlobReference(uri);
            blob.Delete();
        }
    }

    public class BlobContainer
    {
        public String Name { get; set; }
        public String DisplayName { get; set; }
    }

    public class BlobData
    {
        public String Uri { get; set; }
        public String OriginalUri { get; set; }
        public String Memo { get; set; }
        public long Size { get; set; }
        public DateTime Updated { get; set; }
    }
}