﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;

namespace CloudUI.Models
{
    public class DirAlbumImage
    {
        private CloudBlobClient _cli;

        public DirAlbumImage()
        {
            CloudStorageAccount account = CloudStorageAccount.FromConfigurationSetting("MyStorage");
            _cli = account.CreateCloudBlobClient();
            _cli.RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromMilliseconds(500));
        }

        public void UploadImage(String dirName, HttpPostedFile fl, String memo)
        {
            var container = _cli.GetContainerReference("images");
            var dir = container.GetDirectoryReference(dirName);
            var name = String.Format("{0}_{1}",
                    DateTime.UtcNow.ToString("yyyyMMddHHmmss"),
                    Guid.NewGuid().ToString());
            var blob = dir.GetBlobReference(name);
            blob.UploadFromStream(fl.InputStream);
            blob.Properties.ContentType = fl.ContentType;
            blob.SetProperties();
            blob.Metadata.Add("memo", HttpUtility.UrlEncode(memo));
            blob.SetMetadata();
        }

        public IEnumerable<BlobData> GetImages(String dirName)
        {
            var container = _cli.GetContainerReference("images");
            var dir = container.GetDirectoryReference(dirName);
            var opt = new BlobRequestOptions
            {
                BlobListingDetails = BlobListingDetails.Metadata
            };
            return (from b in dir.ListBlobs(opt)
                    let props = ((CloudBlob)b).Properties
                    select new BlobData()
                    {
                        Uri = b.Uri.ToString(),
                        Memo = HttpUtility.UrlDecode(((CloudBlob)b).Metadata["memo"]),
                        Size = props.Length / 1024,
                        Updated = props.LastModifiedUtc.AddHours(9)
                    }).ToList();
        }

        public void DeleteImage(String uri)
        {
            var blob = _cli.GetBlobReference(uri);
            blob.Delete();
        }
    }
}