﻿using System;
using System.IO;
using System.Text;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.StorageClient;

namespace CloudUI.Models
{
    public class DriveData
    {
        private CloudDrive drive;
        public DriveData()
        {
            CloudStorageAccount account = CloudStorageAccount.FromConfigurationSetting("MyStorage");
            var blob_cli = account.CreateCloudBlobClient();
            blob_cli.RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromMilliseconds(500));
            var c = blob_cli.GetContainerReference("drives");
            var page = c.GetPageBlobReference("mydrive");
            LocalResource localCache =
                RoleEnvironment.GetLocalResource("MyCache");
            CloudDrive.InitializeCache(localCache.RootPath,
                localCache.MaximumSizeInMegabytes);
            drive = new CloudDrive(page.Uri, account.Credentials);
        }

        public void UpdateMemo(string name, string body)
        {
            var letter = drive.Mount(10, DriveMountOptions.None);
            var path = String.Format("{0}/{1}", letter, name);
            using (var writer = new StreamWriter(path))
            {
                writer.Write(body);
            }
            drive.Unmount();
        }

        public string GetMemoByName(string name)
        {
            var letter = drive.Mount(10, DriveMountOptions.None);
            var path = String.Format("{0}/{1}", letter, name);
            var result = "";
            if (File.Exists(path))
            {
                using (var reader = new StreamReader(path))
                {
                    result = reader.ReadToEnd();
                }
            }
            drive.Unmount();
            return result;
        }
    }
}