﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;

namespace CloudUI.Models
{
    public class MessageTableAdaptor
    {
        private TableServiceContext _context;

        public MessageTableAdaptor()
        {
            CloudStorageAccount account = CloudStorageAccount.FromConfigurationSetting("MyStorage");
            CloudTableClient table = account.CreateCloudTableClient();
            table.RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromMilliseconds(500));
            _context = table.GetDataServiceContext();
        }

        public IQueryable<Message> Message
        {
            get { return _context.CreateQuery<Message>("Message"); }
        }

        public void AddMessage(String id, String name, String url, String body)
        {
            Message msg = new Message(id)
            {
                Name = HttpUtility.UrlEncode(name),
                Url = url,
                Body = HttpUtility.UrlEncode(body)
            };
            _context.AddObject("Message", msg);
            _context.SaveChanges();
        }

        public void UpdateMessage()
        {
            var message = (from m in this.Message
                           where m.PartitionKey == "General"
                           select m).First();
            message.Url = "http://www.wings.msn.to/";
            _context.UpdateObject(message);
            _context.SaveChanges();
        }

        public void DeleteMessage()
        {
            var message = (from m in this.Message
                           where m.PartitionKey == "General"
                           select m).First();
            _context.DeleteObject(message);
            _context.SaveChanges();
        }

        public IEnumerable<Message> GetBbsMessages(String id)
        {
            var list = this.Message.
                Where(m => m.PartitionKey == id).
                ToList();
            return list.Select(m =>
            {
                m.Name = HttpUtility.UrlDecode(m.Name);
                m.Body = HttpUtility.UrlDecode(m.Body);
                m.Timestamp = m.Timestamp.AddHours(9);
                return m;
            });
        }
    }
}