﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;

namespace CloudUI.Models
{
    public class SnapshotProcess
    {
        private CloudBlobClient _cli;
        private const String CONTAINER_NAME = "images";

        public SnapshotProcess()
        {
            CloudStorageAccount account = CloudStorageAccount.FromConfigurationSetting("MyStorage");
            _cli = account.CreateCloudBlobClient();
            _cli.RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromMilliseconds(500));
        }

        public IEnumerable<BlobData> GetBlobs()
        {
            var container = _cli.GetContainerReference(CONTAINER_NAME);
            return from CloudBlob b in container.ListBlobs(
                    new BlobRequestOptions()
                    {
                        UseFlatBlobListing = true
                    })
                   select new BlobData()
                   {
                       Uri = Path.GetFileName(b.Uri.ToString()),
                       OriginalUri = b.Uri.ToString()
                   };
        }

        public IEnumerable<BlobData> GetSnapshots(String name)
        {
            var container = _cli.GetContainerReference(CONTAINER_NAME);
            return from CloudBlob b in container.ListBlobs(
                    new BlobRequestOptions()
                    {
                        BlobListingDetails = BlobListingDetails.Snapshots,
                        UseFlatBlobListing = true
                    })
                   let st = b.SnapshotTime
                   where st.HasValue &&
                        b.Uri.ToString() == name
                   select new BlobData()
                   {
                       Uri = String.Format("{0}?snapshot={1}",
                            b.Uri.ToString(),
                            st.Value.ToString("yyyy-MM-ddTHH:mm:ss.fffffffZ")),
                       Updated = st.Value

                   };
        }

        public void CreateSnapshotFromBlob(String name)
        {
            var blob = _cli.GetBlobReference(name);
            var snap = blob.CreateSnapshot();
            System.Diagnostics.Trace.TraceInformation(
                "Snapshot Created:" + snap.Uri.ToString());
        }

        public void DeleteBlob(String name)
        {
            var blob = _cli.GetBlobReference(name);
            blob.Delete(new BlobRequestOptions()
            {
                DeleteSnapshotsOption = DeleteSnapshotsOption.IncludeSnapshots
            });
        }

        public void PromoteSnapshot(String baseUri, String snapUri)
        {
            var baseblob = _cli.GetBlobReference(baseUri);
            var snapshot = _cli.GetBlobReference(snapUri);
            baseblob.CopyFromBlob(snapshot);
        }
    }
}