﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;



namespace CloudUI.Models
{
    public class ThumbnailImage
    {
        private CloudBlobClient blob_cli;
        private CloudQueueClient que_cli;

        public ThumbnailImage()
        {
            CloudStorageAccount account = CloudStorageAccount.FromConfigurationSetting("MyStorage");
            blob_cli = account.CreateCloudBlobClient();
            blob_cli.RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromMilliseconds(500));
            que_cli = account.CreateCloudQueueClient();
            que_cli.RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromMilliseconds(500));
        }

        public void UploadImage(String containerName, HttpPostedFile fl, string memo)
        {
            var container = blob_cli.GetContainerReference(containerName);
            var name = String.Format("{0}_{1}",
                    DateTime.UtcNow.ToString("yyyyMMddHHmmss"),
                    Guid.NewGuid().ToString());
            var blob = container.GetBlobReference(name);
            blob.UploadFromStream(fl.InputStream);
            blob.Properties.ContentType = fl.ContentType;
            blob.SetProperties();
            blob.Metadata.Add("memo", HttpUtility.UrlEncode(memo));
            blob.SetMetadata();
            var queue = que_cli.GetQueueReference("thumbnails");
            var msg = new CloudQueueMessage(name);
            queue.AddMessage(msg);
        }

        public IEnumerable<BlobData> GetImages()
        {
            var c_thumb = blob_cli.GetContainerReference("quethumb");
            var c_org = blob_cli.GetContainerReference("queorg");
            var opt = new BlobRequestOptions
            {
                BlobListingDetails = BlobListingDetails.Metadata
            };
            return from b in c_org.ListBlobs(opt)
                   let props = ((CloudBlob)b).Properties
                   select new BlobData()
                   {
                       Uri = c_thumb.GetBlobReference(
                           Path.GetFileName(b.Uri.ToString())).Uri.ToString(),
                       OriginalUri = b.Uri.ToString(),
                       Memo = HttpUtility.UrlDecode(((CloudBlob)b).Metadata["memo"]),
                       Size = props.Length / 1024,
                       Updated = props.LastModifiedUtc.AddHours(9)
                   };
        }

        public void DeleteImage(String uri)
        {
            var blob_name = Path.GetFileName(uri);
            var queue = que_cli.GetQueueReference("delimages");
            var msg = new CloudQueueMessage(blob_name);
            queue.AddMessage(msg);

            var c_org = blob_cli.GetContainerReference("queorg");
            var b_org = c_org.GetBlobReference(blob_name);
            b_org.DeleteIfExists();
        }
    }
}