﻿Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.IO
Imports System.Net
Imports System.Threading
Imports Microsoft.WindowsAzure
Imports Microsoft.WindowsAzure.Diagnostics
Imports Microsoft.WindowsAzure.ServiceRuntime
Imports Microsoft.WindowsAzure.StorageClient

Public Class WorkerRole
    Inherits RoleEntryPoint

    Private blob_cli As CloudBlobClient
    Private que_cli As CloudQueueClient

    Public Overrides Sub Run()
        Dim account = CloudStorageAccount.FromConfigurationSetting("MyStorage")
        blob_cli = account.CreateCloudBlobClient()
        que_cli = account.CreateCloudQueueClient()
        Dim queue = que_cli.GetQueueReference("thumbnails")
        queue.CreateIfNotExist()
        While (True)
            Dim msg = queue.GetMessage()
            Try
                If msg IsNot Nothing Then
                    Me.CreateThumbnail(msg)
                    queue.DeleteMessage(msg)
                    Trace.TraceInformation("Created thumbnail:" & msg.AsString)
                    Thread.Sleep(5000)
                Else
                    Thread.Sleep(10000)
                End If
            Catch e As Exception
                Trace.TraceError("Error Occured:" & e.Message)
                Thread.Sleep(10000)
            End Try
        End While
    End Sub

    Private Sub CreateThumbnail(ByVal msg As CloudQueueMessage)
        Dim org_c = blob_cli.GetContainerReference("queorg")
        Dim thumb_c = blob_cli.GetContainerReference("quethumb")
        Dim blobname = msg.AsString
        Dim org = org_c.GetBlobReference(blobname)
        Dim thumb = thumb_c.GetBlobReference(blobname)

        Using input = New MemoryStream()
            Using output = New MemoryStream()
                org.DownloadToStream(input)
                Using org_img = Image.FromStream(input)
                    Dim t_width = 50
                    Dim t_height = 50
                    If org_img.Width > org_img.Height Then
                        t_height = 50 * org_img.Height / org_img.Width
                    Else
                        t_height = 50 * org_img.Width / org_img.Height
                    End If
                    Using thumb_img = org_img.GetThumbnailImage(t_width, t_height, Nothing, IntPtr.Zero)
                        thumb_img.Save(output, ImageFormat.Jpeg)
                        output.Seek(0, SeekOrigin.Begin)
                    End Using
                End Using
                thumb.UploadFromStream(output)
            End Using
        End Using
    End Sub

    Public Overrides Function OnStart() As Boolean

        DiagnosticMonitor.Start("DiagnosticsConnectionString")
        ' For information on handling configuration changes
        ' see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.
        'AddHandler RoleEnvironment.Changing, AddressOf RoleEnvironmentChanging

        CloudStorageAccount.SetConfigurationSettingPublisher(Function(configName, configSetter) SetConfig(configName, configSetter))
        Return MyBase.OnStart()
    End Function

    Private Function SetConfig(ByVal configName As String, ByVal configSetter As System.Func(Of String, Boolean)) As Object
        configSetter(RoleEnvironment.GetConfigurationSettingValue(configName))
        AddHandler RoleEnvironment.Changed, Function(sender, arg) SetConfigSetter(sender, arg, configName, configSetter)
        Return Nothing
    End Function

    Private Function SetConfigSetter(ByVal sender As Object, ByVal arg As Microsoft.WindowsAzure.ServiceRuntime.RoleEnvironmentChangedEventArgs, ByVal configName As String, ByVal configSetter As System.Func(Of String, Boolean)) As Object
        If arg.Changes.OfType(Of RoleEnvironmentConfigurationSettingChange)().Any(Function(change) (change.ConfigurationSettingName Is configName)) Then
            If Not configSetter(RoleEnvironment.GetConfigurationSettingValue(configName)) Then
                RoleEnvironment.RequestRecycle()
            End If
        End If
        Return Nothing
    End Function

    Private Sub RoleEnvironmentChanging(ByVal sender As Object, ByVal e As RoleEnvironmentChangingEventArgs)

        ' If a configuration setting is changing
        If (e.Changes.Any(Function(change) TypeOf change Is RoleEnvironmentConfigurationSettingChange)) Then
            ' Set e.Cancel to true to restart this role instance
            e.Cancel = True
        End If

    End Sub

End Class
