﻿Imports Microsoft.WindowsAzure
Imports Microsoft.WindowsAzure.StorageClient

Namespace Models
    Public Class AlbumImage
        Private _cli As CloudBlobClient

        Public Sub New()
            Dim account = CloudStorageAccount.FromConfigurationSetting("MyStorage")
            _cli = account.CreateCloudBlobClient()
            _cli.RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromMilliseconds(500))
        End Sub

        Public Function GetContainers() As List(Of BlobContainer)
            Return (From c In _cli.ListContainers("img", ContainerListingDetails.All)
              Select New BlobContainer With
              {
                .Name = c.Name,
                .DisplayName = HttpUtility.UrlDecode(c.Metadata("Name"))
              }).ToList()
        End Function

        Public Sub UploadImage(ByVal containerName As String,
            ByVal fl As HttpPostedFile, ByVal memo As String)
            Dim container = _cli.GetContainerReference(containerName)
            Dim name = String.Format("{0}_{1}",
                DateTime.UtcNow.ToString("yyyyMMddHHmmss"),
                Guid.NewGuid().ToString())
            Dim blob = container.GetBlobReference(name)
            blob.UploadFromStream(fl.InputStream)
            blob.Properties.ContentType = fl.ContentType
            blob.SetProperties()
            blob.Metadata.Add("memo", HttpUtility.UrlEncode(memo))
            blob.SetMetadata()
        End Sub

        Public Function GetImages(ByVal containerName As String) As List(Of BlobData)
            Dim container = _cli.GetContainerReference(containerName)
            Dim opt = New BlobRequestOptions() With
            {
                .BlobListingDetails = BlobListingDetails.Metadata
            }
            Return (From b In container.ListBlobs(opt)
                Let props = DirectCast(b, CloudBlob).Properties
                Select New BlobData() With
                {
                    .Uri = b.Uri.ToString(),
                    .Memo = HttpUtility.UrlDecode(DirectCast(b, CloudBlob).Metadata("memo")),
                    .Size = props.Length / 1024,
                    .Updated = props.LastModifiedUtc.AddHours(9)
                }).ToList()
        End Function

        Public Sub DeleteImage(ByVal uri As String)
            Dim blob = _cli.GetBlobReference(uri)
            blob.Delete()
        End Sub
    End Class

    Public Class BlobContainer
        Public Property Name As String
        Public Property DisplayName As String
    End Class

    Public Class BlobData
        Public Property Uri As String
        Public Property OriginalUri As String
        Public Property Memo As String
        Public Property Size As Long
        Public Property Updated As DateTime
    End Class
End Namespace
