﻿Imports Microsoft.WindowsAzure
Imports Microsoft.WindowsAzure.StorageClient


Namespace Models
    Public Class DirAlbumImage
        Private _cli As CloudBlobClient

        Public Sub New()
            Dim account = CloudStorageAccount.FromConfigurationSetting("MyStorage")
            _cli = account.CreateCloudBlobClient()
            _cli.RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromMilliseconds(500))
        End Sub

        Public Sub UploadImage(ByVal dirName As String,
            ByVal fl As HttpPostedFile, ByVal memo As String)
            Dim container = _cli.GetContainerReference("images")
            Dim dir = container.GetDirectoryReference(dirName)
            Dim name = String.Format("{0}_{1}",
                DateTime.UtcNow.ToString("yyyyMMddHHmmss"),
                Guid.NewGuid().ToString())
            Dim blob = dir.GetBlobReference(name)
            blob.UploadFromStream(fl.InputStream)
            blob.Properties.ContentType = fl.ContentType
            blob.SetProperties()
            blob.Metadata.Add("memo", HttpUtility.UrlEncode(memo))
            blob.SetMetadata()
        End Sub

        Public Function GetImages(ByVal dirName As String) As IEnumerable(Of BlobData)
            Dim container = _cli.GetContainerReference("images")
            Dim dir = container.GetDirectoryReference(dirName)
            Dim opt = New BlobRequestOptions() With
            {
            .BlobListingDetails = BlobListingDetails.Metadata
            }
            Return (From b In dir.ListBlobs(opt)
                Let props = DirectCast(b, CloudBlob).Properties
                Select New BlobData() With
                {
                    .Uri = b.Uri.ToString(),
                    .Memo = HttpUtility.UrlDecode(DirectCast(b, CloudBlob).Metadata("memo")),
                    .Size = props.Length / 1024,
                    .Updated = props.LastModifiedUtc.AddHours(9)
                }).ToList()
        End Function

        Public Sub DeleteImage(ByVal uri As String)
            Dim blob = _cli.GetBlobReference(uri)
            blob.Delete()
        End Sub
    End Class
End Namespace
