﻿Imports Microsoft.WindowsAzure
Imports Microsoft.WindowsAzure.StorageClient

Namespace Models
    Public Class MessageTableAdaptor
        Private _context As TableServiceContext

        Public Sub New()
            Dim account = CloudStorageAccount.FromConfigurationSetting("MyStorage")
            Dim table = account.CreateCloudTableClient()
            table.RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromMilliseconds(500))
            _context = table.GetDataServiceContext()
        End Sub

        Public ReadOnly Property Message As IQueryable(Of Message)
            Get
                Return _context.CreateQuery(Of Message)("Message")
            End Get
        End Property

        Public Sub AddMessage(ByVal id As String, ByVal name As String,
            ByVal url As String, ByVal body As String)
            Dim msg = New Message(id) With
            {
                .Name = HttpUtility.UrlEncode(name),
                .Url = url,
                .Body = HttpUtility.UrlEncode(body)
            }
            _context.AddObject("Message", msg)
            _context.SaveChanges()
        End Sub

        Public Sub UpdateMessage()
            Dim message = (From m In Me.Message
                     Where m.PartitionKey = "General"
                     Select m).First()
            message.Url = "http://www.wings.msn.to/"
            _context.UpdateObject(message)
            _context.SaveChanges()
        End Sub

        Public Sub DeleteMessage()
            Dim message = (From m In Me.Message
                     Where m.PartitionKey = "General"
                     Select m).First()
            _context.DeleteObject(message)
            _context.SaveChanges()
        End Sub

        Public Function GetBbsMessages(ByVal id As String) As IEnumerable(Of Message)
            Dim list = Me.Message.
                Where(Function(m) m.PartitionKey = id).
                ToList()
            Return list.Select(Function(m) DecodeMessage(m))
        End Function

        Private Function DecodeMessage(ByVal m As Message) As Message
            m.Name = HttpUtility.UrlDecode(m.Name)
            m.Body = HttpUtility.UrlDecode(m.Body)
            m.Timestamp = m.Timestamp.AddHours(9)
            Return m
        End Function
    End Class
End Namespace
