﻿Imports System.IO
Imports Microsoft.WindowsAzure
Imports Microsoft.WindowsAzure.StorageClient

Namespace Models
    Public Class SnapshotProcess
        Private _cli As CloudBlobClient
        Private Const CONTAINER_NAME As String = "images"

        Public Sub New()
            Dim account = CloudStorageAccount.FromConfigurationSetting("MyStorage")
            _cli = account.CreateCloudBlobClient()
            _cli.RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromMilliseconds(500))
        End Sub

        Public Function GetBlobs() As IEnumerable(Of BlobData)
            Dim container = _cli.GetContainerReference(CONTAINER_NAME)
            Return From b As CloudBlob In container.ListBlobs(
                    New BlobRequestOptions() With
                    {
                        .UseFlatBlobListing = True
                    })
                   Select New BlobData() With
                   {
                       .Uri = Path.GetFileName(b.Uri.ToString()),
                       .OriginalUri = b.Uri.ToString()
                   }
        End Function

        Public Function GetSnapshots(ByVal name As String) As IEnumerable(Of BlobData)
            Dim container = _cli.GetContainerReference(CONTAINER_NAME)
            Dim list = container.ListBlobs(
                    New BlobRequestOptions() With
                    {
                        .BlobListingDetails = BlobListingDetails.Snapshots,
                        .UseFlatBlobListing = True
                    })
            Return From b As CloudBlob In container.ListBlobs(
                    New BlobRequestOptions() With
                    {
                        .BlobListingDetails = BlobListingDetails.Snapshots,
                        .UseFlatBlobListing = True
                    })
                    Let st = b.SnapshotTime
                    Where st.HasValue And
                        b.Uri.ToString() = name
                    Select New BlobData() With
                    {
                       .Uri = String.Format("{0}?snapshot={1}",
                            b.Uri.ToString(),
                            st.Value.ToString("yyyy-MM-ddTHH:mm:ss.fffffffZ")),
                       .Updated = st.Value
                   }
        End Function

        Public Sub CreateSnapshotFromBlob(ByVal name As String)
            Dim blob = _cli.GetBlobReference(name)
            Dim snap = blob.CreateSnapshot()
            System.Diagnostics.Trace.TraceInformation(
                "Snapshot Created:" & snap.Uri.ToString())
        End Sub

        Public Sub DeleteBlob(ByVal name As String)
            Dim blob = _cli.GetBlobReference(name)
            blob.Delete(New BlobRequestOptions() With
            {
                .DeleteSnapshotsOption = DeleteSnapshotsOption.IncludeSnapshots
            })
        End Sub

        Public Sub PromoteSnapshot(ByVal baseUri As String, ByVal snapUri As String)
            Dim baseblob = _cli.GetBlobReference(baseUri)
            Dim snapshot = _cli.GetBlobReference(snapUri)
            baseblob.CopyFromBlob(snapshot)
        End Sub
    End Class
End Namespace
