﻿Imports System.IO
Imports Microsoft.WindowsAzure.StorageClient
Imports Microsoft.WindowsAzure

Namespace Models
    Public Class ThumbnailImage
        Private blob_cli As CloudBlobClient
        Private que_cli As CloudQueueClient

        Public Sub New()
            Dim account = CloudStorageAccount.FromConfigurationSetting("MyStorage")
            blob_cli = account.CreateCloudBlobClient()
            blob_cli.RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromMilliseconds(500))
            que_cli = account.CreateCloudQueueClient()
            que_cli.RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromMilliseconds(500))
        End Sub

        Public Sub UploadImage(ByVal containerName As String,
            ByVal fl As HttpPostedFile, ByVal memo As String)
            Dim container = blob_cli.GetContainerReference(containerName)
            Dim name = String.Format("{0}_{1}",
                DateTime.UtcNow.ToString("yyyyMMddHHmmss"),
                Guid.NewGuid().ToString())
            Dim blob = container.GetBlobReference(name)
            blob.UploadFromStream(fl.InputStream)
            blob.Properties.ContentType = fl.ContentType
            blob.SetProperties()
            blob.Metadata.Add("memo", HttpUtility.UrlEncode(memo))
            blob.SetMetadata()
            Dim queue = que_cli.GetQueueReference("thumbnails")
            Dim msg = New CloudQueueMessage(name)
            queue.AddMessage(msg)
        End Sub

        Public Function GetImages() As IEnumerable(Of BlobData)
            Dim c_thumb = blob_cli.GetContainerReference("quethumb")
            Dim c_org = blob_cli.GetContainerReference("queorg")
            Dim opt = New BlobRequestOptions() With
            {
                .BlobListingDetails = BlobListingDetails.Metadata
            }
            Return From b In c_org.ListBlobs(opt)
                Let props = DirectCast(b, CloudBlob).Properties
                Select New BlobData() With
                {
                    .Uri = c_thumb.GetBlobReference(
                        Path.GetFileName(b.Uri.ToString())).Uri.ToString(),
                    .OriginalUri = b.Uri.ToString(),
                    .Memo = HttpUtility.UrlDecode(DirectCast(b, CloudBlob).Metadata("memo")),
                    .Size = props.Length / 1024,
                    .Updated = props.LastModifiedUtc.AddHours(9)
                }
        End Function

        Public Sub DeleteImage(ByVal uri As String)
            Dim blob_name = Path.GetFileName(uri)
            Dim queue = que_cli.GetQueueReference("delimages")
            Dim msg = New CloudQueueMessage(blob_name)
            queue.AddMessage(msg)

            Dim c_org = blob_cli.GetContainerReference("queorg")
            Dim b_org = c_org.GetBlobReference(blob_name)
            b_org.DeleteIfExists()
        End Sub
    End Class
End Namespace
